@@ -8,7 +8,7 @@
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
+;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
 ;; This file is distributed in the hope that it will be useful,
@@ -18,8 +18,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -164,7 +164,7 @@ present for backward compatibility."
    '((tramp-parse-netrc "~/.netrc"))))
 
 (defun tramp-efs-file-name-handler (operation &rest args)
-  "Invoke the EFS handler for OPERATION	.
+  "Invoke the EFS handler for OPERATION	.	
 First arg specifies the OPERATION, second args is a list of arguments to
 pass to the OPERATION			.	"
   (save-match-data
@@ -210,7 +210,6 @@ pass to the OPERATION			.	"
     (let ((v (tramp-dissect-file-name filename)))
       (string=
        (tramp-find-method
-	(tramp-file-name-multi-method v)
 	(tramp-file-name-method v)
 	(tramp-file-name-user v)
 	(tramp-file-name-host v))
