@@ -1,8 +1,8 @@
-;;; tramp-ftp.el --- Tramp convenience functions for Ange-FTP -*- coding: iso-8859-1; -*-
+;;; tramp-ftp.el --- Tramp convenience functions for Ange-FTP and EFS -*- coding: iso-8859-1; -*-
 
-;; Copyright (C) 2002, 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
+;; Copyright (C) 2002, 2003, 2004 Free Software Foundation, Inc.
 
-;; Author: Michael Albinus <michael.albinus@gmx.de>
+;; Author: Michael Albinus <Michael.Albinus@alcatel.de>
 ;; Keywords: comm, processes
 
 ;; This file is part of GNU Emacs.
@@ -19,8 +19,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -31,7 +31,9 @@
 
 (require 'tramp)
 
-(eval-when-compile (require 'custom))
+(eval-when-compile
+  (require 'cl)
+  (require 'custom))
 
 ;; Disable Ange-FTP from file-name-handler-alist.
 ;; To handle EFS, the following functions need to be dealt with:
@@ -58,40 +60,8 @@ present for backward compatibility."
 	(a2 (rassq 'ange-ftp-completion-hook-function file-name-handler-alist)))
     (setq file-name-handler-alist
 	  (delete a1 (delete a2 file-name-handler-alist)))))
-
-(eval-after-load "ange-ftp"
-  '(when (functionp 'tramp-disable-ange-ftp)
-     (tramp-disable-ange-ftp)))
-
-;;;###autoload
-(defun tramp-ftp-enable-ange-ftp ()
-  ;; The following code is commented out in Ange-FTP.
-
-  ;;; This regexp takes care of real ange-ftp file names (with a slash
-  ;;; and colon).
-  ;;; Don't allow the host name to end in a period--some systems use /.:
-  (or (assoc "^/[^/:]*[^/:.]:" file-name-handler-alist)
-      (setq file-name-handler-alist
-	    (cons '("^/[^/:]*[^/:.]:" . ange-ftp-hook-function)
-		  file-name-handler-alist)))
-
-  ;;; This regexp recognizes absolute filenames with only one component,
-  ;;; for the sake of hostname completion.
-  (or (assoc "^/[^/:]*\\'" file-name-handler-alist)
-      (setq file-name-handler-alist
-	    (cons '("^/[^/:]*\\'" . ange-ftp-completion-hook-function)
-		  file-name-handler-alist)))
-
-  ;;; This regexp recognizes absolute filenames with only one component
-  ;;; on Windows, for the sake of hostname completion.
-  (and (memq system-type '(ms-dos windows-nt))
-       (or (assoc "^[a-zA-Z]:/[^/:]*\\'" file-name-handler-alist)
-	   (setq file-name-handler-alist
-		 (cons '("^[a-zA-Z]:/[^/:]*\\'" .
-			 ange-ftp-completion-hook-function)
-		       file-name-handler-alist)))))
-
-(add-hook 'tramp-ftp-unload-hook 'tramp-ftp-enable-ange-ftp)
+(tramp-disable-ange-ftp)
+(eval-after-load "ange-ftp" '(tramp-disable-ange-ftp))
 
 ;; Define FTP method ...
 (defcustom tramp-ftp-method "ftp"
@@ -153,7 +123,6 @@ pass to the OPERATION."
   (let ((v (tramp-dissect-file-name filename)))
     (string=
      (tramp-find-method
-      (tramp-file-name-multi-method v)
       (tramp-file-name-method v)
       (tramp-file-name-user v)
       (tramp-file-name-host v))
@@ -171,8 +140,6 @@ pass to the OPERATION."
 ;;   pretended in `tramp-file-name-handler' otherwise.
 ;;   Furthermore, there are no backup files on FTP hosts.
 ;;   Worth further investigations.
-;; * Map /multi:ssh:out@gate:ftp:kai@real.host:/path/to.file
-;;   on Ange-FTP gateways.
 
 ;;; arch-tag: 759fb338-5c63-4b99-bd36-b4d59db91cff
 ;;; tramp-ftp.el ends here
