@@ -1,8 +1,8 @@
 ;;; tramp-smb.el --- Tramp access functions for SMB servers -*- coding: iso-8859-1; -*-
 
-;; Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
+;; Copyright (C) 2002, 2003, 2004 Free Software Foundation, Inc.
 
-;; Author: Michael Albinus <michael.albinus@gmx.de>
+;; Author: Michael Albinus <Michael.Albinus@alcatel.de>
 ;; Keywords: comm, processes
 
 ;; This file is part of GNU Emacs.
@@ -19,8 +19,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -33,10 +33,7 @@
 ;; Pacify byte-compiler
 (eval-when-compile
   (require 'cl)
-  (require 'custom)
-  ;; Emacs 19.34 compatibility hack -- is this needed?
-  (or (>= emacs-major-version 20)
-      (load "cl-seq")))
+  (require 'custom))
 
 ;; Avoid byte-compiler warnings if the byte-compiler supports this.
 ;; Currently, XEmacs supports this.
@@ -46,6 +43,11 @@
       (defalias 'warnings 'identity) ; Pacify Emacs byte-compiler
       (byte-compiler-options (warnings (- unused-vars))))))
 
+;; XEmacs byte-compiler raises warning abouts `last-coding-system-used'.
+(eval-when-compile
+  (unless (boundp 'last-coding-system-used)
+    (defvar last-coding-system-used nil)))
+
 ;; Define SMB method ...
 (defcustom tramp-smb-method "smb"
   "*Method to connect SAMBA and M$ SMB servers."
@@ -58,7 +60,12 @@
 ;; Add a default for `tramp-default-method-alist'. Rule: If there is
 ;; a domain in USER, it must be the SMB method.
 (add-to-list 'tramp-default-method-alist
-	     (list "" "%" tramp-smb-method))
+	     `(nil "%" ,tramp-smb-method))
+
+;; Add a default for `tramp-default-user-alist'. Rule: For the SMB method,
+;; the anonymous user is chosen.
+(add-to-list 'tramp-default-user-alist
+	     `(,tramp-smb-method nil ""))
 
 ;; Add completion function for SMB method.
 (tramp-set-completion-function
@@ -70,7 +77,7 @@
   :group 'tramp
   :type 'string)
 
-(defconst tramp-smb-prompt "^smb: .+> \\|^\\s-+Server\\s-+Comment$"
+(defconst tramp-smb-prompt "^smb: \\S-+> \\|^\\s-+Server\\s-+Comment$"
   "Regexp used as prompt in smbclient.")
 
 (defconst tramp-smb-errors
@@ -167,7 +174,7 @@ This variable is local to each buffer.")
     (set-file-modes . tramp-smb-not-handled)
     (set-visited-file-modtime . tramp-smb-not-handled)
     (shell-command . tramp-smb-not-handled)
-    (substitute-in-file-name . tramp-smb-handle-substitute-in-file-name)
+    ;; `substitute-in-file-name' performed by default handler
     (unhandled-file-name-directory . tramp-handle-unhandled-file-name-directory)
     (vc-registered . tramp-smb-not-handled)
     (verify-visited-file-modtime . tramp-smb-not-handled)
@@ -181,7 +188,6 @@ Operations not mentioned here will be handled by the default Emacs primitives.")
   (let ((v (tramp-dissect-file-name filename)))
     (string=
      (tramp-find-method
-      (tramp-file-name-multi-method v)
       (tramp-file-name-method v)
       (tramp-file-name-user v)
       (tramp-file-name-host v))
@@ -237,53 +243,55 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	    (unless share
 	      (error "Target `%s' must contain a share name" filename))
 	    (tramp-smb-maybe-open-connection user host share)
-	    (tramp-message-for-buffer
-	     nil tramp-smb-method user host
+	    (tramp-message-for-buffer tramp-smb-method user host
 	     5 "Copying file %s to file %s..." filename newname)
 	    (if (tramp-smb-send-command
 		 user host (format "put %s \"%s\"" filename file))
-		(tramp-message-for-buffer
-		 nil tramp-smb-method user host
+		(tramp-message-for-buffer tramp-smb-method user host
 		 5 "Copying file %s to file %s...done" filename newname)
 	      (error "Cannot copy `%s'" filename))))))))
 
 (defun tramp-smb-handle-delete-directory (directory)
   "Like `delete-directory' for tramp files."
   (setq directory (directory-file-name (expand-file-name directory)))
-  (when (file-exists-p directory)
-    (with-parsed-tramp-file-name directory nil
-      (save-excursion
-	(let ((share (tramp-smb-get-share localname))
-	      (dir (tramp-smb-get-localname (file-name-directory localname) t))
-	      (file (file-name-nondirectory localname)))
-	  (tramp-smb-maybe-open-connection user host share)
-	  (if (and
-	       (tramp-smb-send-command user host (format "cd \"%s\"" dir))
-	       (tramp-smb-send-command user host (format "rmdir \"%s\"" file)))
-	      ;; Go Home
-	      (tramp-smb-send-command user host (format "cd \\"))
-	    ;; Error
+  (unless (file-exists-p directory)
+    (error "Cannot delete non-existing directory `%s'" directory))
+  (with-parsed-tramp-file-name directory nil
+    (save-excursion
+      (let ((share (tramp-smb-get-share localname))
+	    (dir (tramp-smb-get-localname (file-name-directory localname) t))
+	    (file (file-name-nondirectory localname)))
+	(tramp-smb-maybe-open-connection user host share)
+	(if (and
+	     (tramp-smb-send-command user host (format "cd \"%s\"" dir))
+	     (tramp-smb-send-command user host (format "rmdir \"%s\"" file)))
+	    ;; Go Home
 	    (tramp-smb-send-command user host (format "cd \\"))
-	    (error "Cannot delete directory `%s'" directory)))))))
+	  ;; Error
+	  (tramp-smb-send-command user host (format "cd \\"))
+	  (error "Cannot delete directory `%s'" directory))))))
 
 (defun tramp-smb-handle-delete-file (filename)
   "Like `delete-file' for tramp files."
   (setq filename (expand-file-name filename))
-  (when (file-exists-p filename)
-    (with-parsed-tramp-file-name filename nil
-      (save-excursion
-	(let ((share (tramp-smb-get-share localname))
-	      (dir (tramp-smb-get-localname (file-name-directory localname) t))
-	      (file (file-name-nondirectory localname)))
-	  (tramp-smb-maybe-open-connection user host share)
-	  (if (and
-	       (tramp-smb-send-command user host (format "cd \"%s\"" dir))
-	       (tramp-smb-send-command user host (format "rm \"%s\"" file)))
-	      ;; Go Home
-	      (tramp-smb-send-command user host (format "cd \\"))
-	    ;; Error
+  (unless (file-exists-p filename)
+    (error "Cannot delete non-existing file `%s'" filename))
+  (with-parsed-tramp-file-name filename nil
+    (save-excursion
+      (let ((share (tramp-smb-get-share localname))
+	    (dir (tramp-smb-get-localname (file-name-directory localname) t))
+	    (file (file-name-nondirectory localname)))
+	(unless (file-exists-p filename)
+	  (error "Cannot delete non-existing file `%s'" filename))
+	(tramp-smb-maybe-open-connection user host share)
+	(if (and
+	     (tramp-smb-send-command user host (format "cd \"%s\"" dir))
+	     (tramp-smb-send-command user host (format "rm \"%s\"" file)))
+	    ;; Go Home
 	    (tramp-smb-send-command user host (format "cd \\"))
-	    (error "Cannot delete file `%s'" filename)))))))
+	  ;; Error
+	  (tramp-smb-send-command user host (format "cd \\"))
+	  (error "Cannot delete file `%s'" filename))))))
 
 (defun tramp-smb-handle-directory-files
   (directory &optional full match nosort)
@@ -319,11 +327,11 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
   (mapcar
    (lambda (x)
      ;; We cannot call `file-attributes' for backward compatibility reasons.
-     ;; Its optional parameter ID-FORMAT is introduced with Emacs 22.
+     ;; Its optional parameter ID-FORMAT is introduced with Emacs 21.4.
      (cons x (tramp-smb-handle-file-attributes
 	(if full x (concat (file-name-as-directory directory) x)) id-format)))
    (directory-files directory full match nosort)))
-
+ 
 (defun tramp-smb-handle-file-attributes (filename &optional id-format)
   "Like `file-attributes' for tramp files."
   (with-parsed-tramp-file-name filename nil
@@ -336,7 +344,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	     (uid (if (and id-format (equal id-format 'string)) "nobody" -1))
 	     (gid (if (and id-format (equal id-format 'string)) "nogroup" -1))
 	     (inode (tramp-smb-get-inode share file))
-	     (device (tramp-get-device nil tramp-smb-method user host)))
+	     (device (tramp-get-device tramp-smb-method user host)))
 
 	; check result
 	(when entry
@@ -387,14 +395,12 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	    (tmpfil (tramp-make-temp-file)))
 	(unless (file-exists-p filename)
 	  (error "Cannot make local copy of non-existing file `%s'" filename))
-	(tramp-message-for-buffer
-	 nil tramp-smb-method user host
+	(tramp-message-for-buffer tramp-smb-method user host
 	 5 "Fetching %s to tmp file %s..." filename tmpfil)
 	(tramp-smb-maybe-open-connection user host share)
 	(if (tramp-smb-send-command
 	     user host (format "get \"%s\" %s" file tmpfil))
-	    (tramp-message-for-buffer
-	     nil tramp-smb-method user host
+	    (tramp-message-for-buffer tramp-smb-method user host
 	     5 "Fetching %s to tmp file %s...done" filename tmpfil)
 	  (error "Cannot make local copy of file `%s'" filename))
 	tmpfil))))
@@ -461,7 +467,7 @@ WILDCARD and FULL-DIRECTORY-P are not handled."
 	     (entries (tramp-smb-get-file-entries user host share file)))
 
 	;; Delete dummy "" entry, useless entries
-	(setq entries
+	(setq entries 
 	      (if (file-directory-p filename)
 		  (delq (assoc "" entries) entries)
 		;; We just need the only and only entry FILENAME.
@@ -557,25 +563,16 @@ WILDCARD and FULL-DIRECTORY-P are not handled."
 	  (let ((share (tramp-smb-get-share localname))
 		(file (tramp-smb-get-localname localname t)))
 	    (tramp-smb-maybe-open-connection user host share)
-	    (tramp-message-for-buffer
-	     nil tramp-smb-method user host
+	    (tramp-message-for-buffer tramp-smb-method user host
 	     5 "Copying file %s to file %s..." filename newname)
 	    (if (tramp-smb-send-command
 		 user host (format "put %s \"%s\"" filename file))
-		(tramp-message-for-buffer
-		 nil tramp-smb-method user host
+		(tramp-message-for-buffer tramp-smb-method user host
 		 5 "Copying file %s to file %s...done" filename newname)
 	      (error "Cannot rename `%s'" filename)))))))
 
   (delete-file filename))
 
-(defun tramp-smb-handle-substitute-in-file-name (filename)
-  "Like `handle-substitute-in-file-name' for tramp files.
-Catches errors for shares like \"C$/\", which are common in Microsoft Windows."
-  (condition-case nil
-      (tramp-run-real-handler 'substitute-in-file-name (list filename))
-    (error filename)))
-
 (defun tramp-smb-handle-write-region
   (start end filename &optional append visit lockname confirm)
   "Like `write-region' for tramp files."
@@ -593,6 +590,13 @@ Catches errors for shares like \"C$/\", which are common in Microsoft Windows."
       (let ((share (tramp-smb-get-share localname))
 	    (file (tramp-smb-get-localname localname t))
 	    (curbuf (current-buffer))
+	    ;; We use this to save the value of `last-coding-system-used'
+	    ;; after writing the tmp file.  At the end of the function,
+	    ;; we set `last-coding-system-used' to this saved value.
+	    ;; This way, any intermediary coding systems used while
+	    ;; talking to the remote shell or suchlike won't hose this
+	    ;; variable.  This approach was snarfed from ange-ftp.el.
+	    coding-system-used
 	    tmpfil)
 	;; Write region into a tmp file.
 	(setq tmpfil (tramp-make-temp-file))
@@ -604,15 +608,16 @@ Catches errors for shares like \"C$/\", which are common in Microsoft Windows."
 	 (if confirm ; don't pass this arg unless defined for backward compat.
 	     (list start end tmpfil append 'no-message lockname confirm)
 	   (list start end tmpfil append 'no-message lockname)))
+	;; Now, `last-coding-system-used' has the right value.  Remember it.
+	(when (boundp 'last-coding-system-used)
+	  (setq coding-system-used last-coding-system-used))
 
 	(tramp-smb-maybe-open-connection user host share)
-	(tramp-message-for-buffer
-	 nil tramp-smb-method user host
+	(tramp-message-for-buffer tramp-smb-method user host
 	 5 "Writing tmp file %s to file %s..." tmpfil filename)
 	(if (tramp-smb-send-command
 	     user host (format "put %s \"%s\"" tmpfil file))
-	    (tramp-message-for-buffer
-	     nil tramp-smb-method user host
+	    (tramp-message-for-buffer tramp-smb-method user host
 	     5 "Writing tmp file %s to file %s...done" tmpfil filename)
 	  (error "Cannot write `%s'" filename))
 
@@ -621,7 +626,10 @@ Catches errors for shares like \"C$/\", which are common in Microsoft Windows."
 	  (error "Buffer has changed from `%s' to `%s'"
 		 curbuf (current-buffer)))
 	(when (eq visit t)
-	  (set-visited-file-modtime))))))
+	  (set-visited-file-modtime))
+	;; Make `last-coding-system-used' have the right value.
+	(when (boundp 'last-coding-system-used)
+	  (setq last-coding-system-used coding-system-used))))))
 
 
 ;; Internal file name functions
@@ -669,7 +677,7 @@ Result is a list of (LOCALNAME MODE SIZE MONTH DAY TIME YEAR)."
 			   (regexp-quote (match-string 1 localname)))
 		      ""))
 	    res entry)
-	(set-buffer (tramp-get-buffer nil tramp-smb-method user host))
+	(set-buffer (tramp-get-buffer tramp-smb-method user host))
 	(if (and (not share) tramp-smb-share-cache)
 	    ;; Return cached shares
 	    (setq res tramp-smb-share-cache)
@@ -726,7 +734,7 @@ Result is a list of (LOCALNAME MODE SIZE MONTH DAY TIME YEAR)."
 ;; \s-                                    - space delimeter
 ;; \w\{3,3\}                              - month
 ;; \s-                                    - space delimeter
-;; [ 19][0-9]                             - day
+;; [ 12][0-9]                             - day
 ;; \s-                                    - space delimeter
 ;; [0-9]\{2,2\}:[0-9]\{2,2\}:[0-9]\{2,2\} - time
 ;; \s-                                    - space delimeter
@@ -865,9 +873,9 @@ If it doesn't exist, generate a new one."
 Erases temporary buffer before sending the command.  Returns nil if
 there has been an error message from smbclient."
   (save-excursion
-    (set-buffer (tramp-get-buffer nil tramp-smb-method user host))
+    (set-buffer (tramp-get-buffer tramp-smb-method user host))
     (erase-buffer)
-    (tramp-send-command nil tramp-smb-method user host command nil t)
+    (tramp-send-command tramp-smb-method user host command nil t)
     (tramp-smb-wait-for-output user host)))
 
 (defun tramp-smb-maybe-open-connection (user host share)
@@ -876,9 +884,9 @@ Does not do anything if a connection is already open, but re-opens the
 connection if a previous connection has died for some reason."
   (let ((process-connection-type tramp-process-connection-type)
 	(p (get-buffer-process
-	    (tramp-get-buffer nil tramp-smb-method user host))))
+	    (tramp-get-buffer tramp-smb-method user host))))
     (save-excursion
-      (set-buffer (tramp-get-buffer nil tramp-smb-method user host))
+      (set-buffer (tramp-get-buffer tramp-smb-method user host))
       ;; Check whether it is still the same share
       (unless (and p (processp p) (string-equal tramp-smb-share share))
 	(when (and p (processp p))
@@ -905,12 +913,8 @@ then sends the password to the remote host.
 
 Domain names in USER and port numbers in HOST are acknowledged."
 
-  (when (and (fboundp 'executable-find)
-	     (not (funcall 'executable-find tramp-smb-program)))
-    (error "Cannot find command %s in %s" tramp-smb-program exec-path))
-
   (save-match-data
-    (let* ((buffer (tramp-get-buffer nil tramp-smb-method user host))
+    (let* ((buffer (tramp-get-buffer tramp-smb-method user host))
 	   (real-user user)
 	   (real-host host)
 	   domain port args)
@@ -936,17 +940,12 @@ Domain names in USER and port numbers in HOST are acknowledged."
       (when port   (setq args (append args (list "-p" port))))
 
       ; OK, let's go
-      (tramp-pre-connection nil tramp-smb-method user host tramp-chunksize)
-      (tramp-message 7 "Opening connection for //%s@%s/%s..."
-		     user host (or share ""))
+      (tramp-pre-connection tramp-smb-method user host)
+      (tramp-message
+       7 "Opening connection for //%s@%s/%s..." user host (or share ""))
 
       (let* ((default-directory (tramp-temporary-file-directory))
-	     ;; If we omit the conditional here, then we would use
-	     ;; `undecided-dos' in some cases.  With the conditional,
-	     ;; we use nil in these cases.  Which one is right?
-	     (coding-system-for-read (unless (and (not (featurep 'xemacs))
-						  (> emacs-major-version 20))
-				       tramp-dos-coding-system))
+	     (coding-system-for-read nil)
 	     (p (apply #'start-process (buffer-name buffer) buffer
 		       tramp-smb-program args)))
 
@@ -982,7 +981,7 @@ Returns nil if an error message has appeared."
     (while (and (not found) (not err))
 
       ;; Accept pending output.
-      (tramp-accept-process-output proc)
+      (accept-process-output proc)
 
       ;; Search for prompt.
       (goto-char (point-min))
@@ -995,7 +994,7 @@ Returns nil if an error message has appeared."
     ;; Add output to debug buffer if appropriate.
     (when tramp-debug-buffer
       (append-to-buffer
-       (tramp-get-debug-buffer nil tramp-smb-method user host)
+       (tramp-get-debug-buffer tramp-smb-method user host)
        (point-min) (point-max)))
 
     ;; Return value is whether no error message has appeared.
@@ -1031,6 +1030,45 @@ Return the difference in the format of a time value."
 	  (- (+ (if borrow 65536 0) (cadr t1)) (cadr t2)))))
 
 
+;; `PC-do-completion' touches the returning "$$" by `substitute-in-file-name'.
+;; Must be corrected.
+
+(defadvice PC-do-completion (around tramp-smb-advice-PC-do-completion activate)
+  "Changes \"$\" back to \"$$\" in minibuffer."
+  (if (funcall PC-completion-as-file-name-predicate)
+
+      (progn
+	;; Substitute file names
+	(let* ((beg (or (and (functionp 'minibuffer-prompt-end) ; Emacs 21
+			     (funcall 'minibuffer-prompt-end))
+			(point-min)))
+	       (end (point-max))
+	       (str (substitute-in-file-name (buffer-substring beg end))))
+	  (delete-region beg end)
+	  (insert str)
+	  (ad-set-arg 2 (point)))
+
+	;; Do `PC-do-completion' without substitution
+	(let* (save)
+	  (fset 'save (symbol-function 'substitute-in-file-name))
+	  (fset 'substitute-in-file-name (symbol-function 'identity))
+	  ad-do-it
+	  (fset 'substitute-in-file-name (symbol-function 'save)))
+
+	;; Expand "$"
+	(let* ((beg (or (and (functionp 'minibuffer-prompt-end) ; Emacs 21
+			     (funcall 'minibuffer-prompt-end))
+			(point-min)))
+	       (end (point-max))
+	       (str (buffer-substring beg end)))
+	  (delete-region beg end)
+	  (insert (if (string-match "\\(\\$\\)\\(/\\|$\\)" str)
+		      (replace-match "$$" nil nil str 1)
+		    str))))
+
+    ;; No file names. Behave unchanged.
+    ad-do-it))
+
 (provide 'tramp-smb)
 
 ;;; TODO:
